# m3.py - Steve Ellingson - Virginia Tech - 2010 Nov 06
# ---
# Reads a sky map file in the form l [rad], b [rad], T [K] (typically from m1)
#   Writes same map in different coordinates to file "m3.py"
# Requires PyEphem (see: http://rhodesmill.org/pyephem)
# ---
# usage:
#   $ python m3.py <filename> <coord> <otime> <obs>
#     <filename>: filename for sky temp map in Galactic coordinates (typically from m1)
#     <coord>:    Coordinate system for output    
#                    "radec": RA/DEC  
#                    "azel":  AZ/EL.  AZ=0 is N, AZ=90deg is E, EL=0 is horizon ( "elevation" = "altitude" )
#     <otime>:    time (UT), in any format recognized by the PyEphem "ephem.date()" function
#                   (see: http://rhodesmill.org/pyephem/quick.html#dates)
#                   Output is precessed to this time 
#                 Surround with double quotes if argument contains spaces
#     <obs>:      observer location, required if <coord>="azel" 
#                 Format is <lon> <lat> <el>
#                   <lon> longitude [deg], in any format PyEphem will recognize
#                   <lat> latitude [deg], in any format PyEphem will recognize
#                   <el>  elevation [m]
#                 Surround with double quotes if argument contains spaces
#   Output goes to m3.dat.  Line format is <c1> <c2> <T> where
#     <c1> is RA  [h]   if <coord>="radec", or AZ [deg] if <coord>="azel"
#     <c2> is DEC [deg] if <coord>="radec", or EL [deg] if <coord>="azel"
#     <T>  is T [K] 
# ---
# Examples:
#   $ python m3.py m1.dat radec "2010/11/06 05:16:30" 
#   $ python m3.py m1.dat azel  "2010/11/06 05:16:30" "-107.628 +34.070 2124.0"

import string
import sys     # required for reading command line args
import ephem

# Initializing things
PI = 3.14159265
TINY = 1.0e-20
x  = ephem.FixedBody() # doing this once and overwriting with each new point

# ---------------------------------
# Processing command line arguments
# ---------------------------------

# Check for required command line arguments 
if len(sys.argv)<4: 
    print 'FATAL: Proper usage is "m3.py <filename> <coord> <otime> <obs>".'
    exit()
else:               
    filename =       string.strip(sys.argv[1]) 
    coord    =       string.strip(sys.argv[2])
    otime    =       string.strip(sys.argv[3])
    obs      = ""
    obs_lon  = ""
    obs_lat  = ""
    obs_el   = ""

if (coord=='azel'):
    if len(sys.argv)<5:
        print 'FATAL: <obs> argument required if <coord>=="azel"'
        exit()
    else:
        obs = string.strip(sys.argv[4])
        obs_args = string.split(obs,' ',3)
        obs_lon = string.strip(obs_args[0]);
        obs_lat = string.strip(obs_args[1]);
        obs_el  = string.strip(obs_args[2]);

# Convert "otime" to a PyEphem date
d = ephem.Date(otime)

# Convert "obs" to a PyEphem observer
if len(obs)>0:
    o = ephem.Observer()
    o.long      = obs_lon
    o.lat       = obs_lat
    o.elevation = float(obs_el)
    o.date      = d

# show what we figured out from the command line arguments
print 'filename = "'+filename+'"'
print 'coord    = "'+coord+'"'
if (coord=='azel'):
    print 'o.date      = '+str(o.date)+' UT'
    print 'o.long      = '+str(o.long)
    print 'o.lat       = '+str(o.lat)
    print 'o.elevation = '+str(o.elevation)+' [m]'
else:
    print 'd        = '+str(d)+' UT'

#exit()

# Read the input file into a string
fp = open(filename,'r')
in_file = fp.read()
fp.close()

# Open the output file
fp = open('m3.dat','w');

# Parse the string into the data table
l = [] # this becomes a list of l [rad]
b = [] # this becomes a list of b [rad]
T = [] # this becomes a list of T [K]

line = in_file.splitlines()   # split file into lines

for i in range(len(line)):
    #print i, line[i]
    column = string.split(line[i],' ',3) # split line into columns
    #print i,'|'+column[0]+'|'+column[1]+'|'
    l = float(string.strip(column[0]));
    b = float(string.strip(column[1]));
    T = float(string.strip(column[2]));

    # get location in RA/DEC for current epoch
    pos = ephem.Equatorial( ephem.Galactic( l, b, epoch=d ) ) 
    #fp.write(str(pos.ra)+' '+str(pos.dec)+' '+str(T)+'\n')

    if coord=='radec':
      ras  = "%f" % ((pos.ra )* 12.0/PI) # get RA as a string representation of a simple float
      decs = "%f" % ((pos.dec)*180.0/PI) # get DEC as a string representation of a simple float
      fp.write(ras+' '+decs+' '+str(T)+'\n')

    if coord=='azel':
      x._ra    = pos.ra     # Defining FixedBody object attributes
      x._dec   = pos.dec
      x._epoch = pos.epoch
      x.compute(o)          # updates az and alt using defined observer "o"
      azs  = "%f" % ((x.az )*180.0/PI) # get AZ as a string representation of a simple float
      els  = "%f" % ((x.alt)*180.0/PI) # get EL as a string representation of a simple float
      fp.write(azs+' '+els+' '+str(T)+'\n')     

# Close the output file
fp.close()











