// m4.c: Steve Ellingson, Virginia Tech, 2010 Nov 07
// ---
// Reads sky temp file having format <c1> <c2> <T> where 
//   <c1>,<c2> are coordinates; e.g., l,b or RA,DEC or AZ,EL
//   T is temp
//   (presumed to be from m3; i.e., m3.dat.  Could also be from m1.)
// Re-grids the data (using a binning+averaging technique) to the specified grid
// ---
// USAGE:
//   m4 <in_filename> <format> <c1min> <c1res> <c1max> <c2min> <c2res> <c2max>
//     <in_filename>:  Typically, "m1.dat"
//     <format>:       0 = list format.  Same as input, but now pixel centers are a simple
//                       raster scan over <c1> and <c2>
//                     1 = matrix format.  Each line is constant <c2>, columns are for varying <c1>,
//                       coordinates themselves are not included.  Min to max in each dimension.
//                       (Conveninent for Gnuplot display)
//     <c1min>:        Min value of coordinate <c1>; defining this edge of the map.
//     <c1res>:        Pixel dimension in coordinate <c1>
//     <c1max>:        Max value of coordinate <c1>; defining this edge of the map.
//     <c2min>, <c2res>, <c2max> defined similarly
//   output goes to m4.dat
// EXAMPLE: Assuming m3.dat is in RA/DEC, an all-sky regridding to 1 x 1 degree would be:
//   $ ./m4 m3.dat 0 0.0 0.066667 24.0 -90.0 1.0 90.0

#include <math.h>
#include <stdio.h>
#include <string.h>

#define PI 3.14159265
#define IC1MAX 360 /* max number of pixels in c1 coordinate */
#define IC2MAX 360 /* max number of pixels in c2 coordinate */

#define ME "m4"

main ( int narg, char *argv[] ) {

  char in_filename[256];
  int eFormat;
  float c1min, c1res, c1max;
  float c2min, c2res, c2max;

  FILE *fp;
  float c1,c2,t;

  int ic1;
  int ic2;
  float map[IC1MAX][IC2MAX];
  int count[IC1MAX][IC2MAX];

  int ic1max;
  int ic2max;

  /*==================*/
  /*=== Initialize ===*/
  /*==================*/
    
  /* First, announce thyself */
  printf("%s: Starting\n",ME);
  printf("%s: Compiled for IC1MAX=%d, IC2MAX=%d\n",ME,IC1MAX,IC2MAX);

  /* initializing */
  for (ic1=0; ic1<IC1MAX; ic1++) {
    for (ic2=0; ic2<IC2MAX; ic2++) {
      map[ic1][ic2] = 0.0;
      count[ic1][ic2] = 0;
      }
    }

  /* clearing this just in case */
  sprintf(in_filename,"");

  if (narg>1) {
      sscanf(argv[1],"%s",in_filename);
    } else {
      printf("%s: FATAL: in_filename not specified\n",ME);
      return;
    }
  if (narg<9) {
    printf("%s: FATAL: required arguments missing.\n",ME);
    return;
    }
  sscanf(argv[2],"%d",&eFormat);
  sscanf(argv[3],"%f",&c1min);
  sscanf(argv[4],"%f",&c1res);
  sscanf(argv[5],"%f",&c1max);
  sscanf(argv[6],"%f",&c2min);
  sscanf(argv[7],"%f",&c2res);
  sscanf(argv[8],"%f",&c2max);

  printf("%s: in_filename = '%s'\n",ME,in_filename);
  printf("%s: eFormat = %d\n",ME,eFormat);
  printf("%s: c1min   = %f\n",ME,c1min);
  printf("%s: c1res   = %f\n",ME,c1res);
  printf("%s: c1max   = %f\n",ME,c1max);
  printf("%s: c2min   = %f\n",ME,c2min);
  printf("%s: c2res   = %f\n",ME,c2res);
  printf("%s: c2max   = %f\n",ME,c2max);

  if ( floor( (c1max-c1min)/c1res ) > IC1MAX ) {
    printf("%s: FATAL: Required grid too large in c1.\n",ME);
    return;
    }
  if ( floor( (c2max-c2min)/c2res ) > IC2MAX ) {
    printf("%s: FATAL: Required grid too large in c2.\n",ME);
    return;
    }
  
  ic1max = floor( (c1max-c1min)/c1res );
  ic2max = floor( (c2max-c2min)/c2res );
  printf("%s: ic1max = %d\n",ME,ic1max);
  printf("%s: ic2max = %d\n",ME,ic2max);

  fp = fopen(in_filename,"r");
  if (!fp) {
    printf("%s: FATAL: Couldn't open '%s'\n",ME,in_filename);
    return;
    }

  while (!feof(fp)) {

    fscanf(fp,"%f %f %f",&c1,&c2,&t);

    if ( (c1>=c1min) && (c1<=c1max) && (c2>=c2min) && (c2<=c2max) ) {

      ic1 = (int) floor( (c1-c1min)/c1res );
      ic2 = (int) floor( (c2-c2min)/c2res );
      map[ic1][ic2] += t;
      count[ic1][ic2]++;

      } /* if ( (c1>=c1min) ... */

    } /* while (!feof(fp)) */

  fclose(fp);

  /* do averaging */
  for (ic1=0; ic1<ic1max; ic1++) {
    for (ic2=0; ic2<ic2max; ic2++) {
      if (count[ic1][ic2]<1) count[ic1][ic2]=1;
      map[ic1][ic2] /= count[ic1][ic2];
      }
    }

  /* open output file */
  fp = fopen("m4.dat","w");
  if (!fp) {
    printf("%s: FATAL: Couldn't open m4.dat\n",ME);
    return;
    }

  /* write it out */
  if (eFormat==0) { /* list */
    for (ic2=0; ic2<ic2max; ic2++) {
      for (ic1=0; ic1<ic1max; ic1++) {
        fprintf(fp,"%f %f %f\n", ic1*c1res+c1min, ic2*c2res+c2min, map[ic1][ic2] );
        }
      }
    }
  if (eFormat==1) { /* matrix */
    for (ic2=0; ic2<ic2max; ic2++) {
      for (ic1=0; ic1<ic1max; ic1++) {
        fprintf(fp,"%f ", map[ic1][ic2] );
        }
      fprintf(fp,"\n" );
      }
    }

  fclose(fp);

  return;
  } /* main() */


