// m5.c: Steve Ellingson, Virginia Tech, 2010 Nov 08
// ---
// Reads sky temp file having format <az> <alt> <T> where 
//   <az>,<alt> are coordinates [degrees]
//   T is temp [K]
//   (presumed to be from m3; i.e., m3.dat.)
// Coordinate conversion to u-v (i.e., direction cosines) projection onto ground.
// Re-grids the data (using a binning+averaging technique) to the specified resolution.
// ---
// USAGE:
//   m5 <in_filename> <format> <res> 
//     <in_filename>:  Typically, "m3.dat"
//     <format>:       0 = list format.  Same as input, but now pixel centers are a simple
//                       raster scan over u and v
//                     1 = matrix format.  Each line is constant v, columns are for varying u,
//                       coordinates themselves are not included.  Min to max in each dimension.
//                       (Conveninent for Gnuplot display)
//     <res>:          Pixel dimension in degrees
//   output goes to m5.dat
// EXAMPLE: Assuming m3.dat is in AZ/ALT, a regridding to u-v with 1 x 1 degree resolution woud be:
//   $ ./m5 m3.dat 0 0.01 

#include <math.h>
#include <stdio.h>
#include <string.h>

#define SIDE_MAX 360           /* max number of pixels in u or v */
#define PI  3.14159265
#define DTR (3.14159265/180.0)
#define ME "m5"

main ( int narg, char *argv[] ) {

  char in_filename[256];
  int eFormat;
  float res;

  FILE *fp;
  float az,alt,t;

  float u,v;
  int iu;
  int iv;
  float map[SIDE_MAX][SIDE_MAX];
  int count[SIDE_MAX][SIDE_MAX];

  int iumax;
  int ivmax;

  float theta;
  float phi;

  /*==================*/
  /*=== Initialize ===*/
  /*==================*/
    
  /* First, announce thyself */
  printf("%s: Starting\n",ME);
  printf("%s: Compiled for SIDE_MAX=%d\n",ME,SIDE_MAX);

  /* initializing */
  for (iu=0; iu<SIDE_MAX; iu++) {
    for (iv=0; iv<SIDE_MAX; iv++) {
      map[iu][iv] = 0.0;
      count[iu][iv] = 0;
      }
    }

  /* clearing this just in case */
  sprintf(in_filename,"");

  if (narg>1) {
      sscanf(argv[1],"%s",in_filename);
    } else {
      printf("%s: FATAL: in_filename not specified\n",ME);
      return;
    }
  if (narg<4) {
    printf("%s: FATAL: required arguments missing.\n",ME);
    return;
    }
  sscanf(argv[2],"%d",&eFormat);
  sscanf(argv[3],"%f",&res);

  printf("%s: in_filename = '%s'\n",ME,in_filename);
  printf("%s: eFormat     = %d\n",ME,eFormat);
  printf("%s: res         = %f\n",ME,res);

  if ( floor(2.0/res) > SIDE_MAX ) {
    printf("%s: FATAL: Required grid too large.\n",ME);
    return;
    }
  
  iumax = floor( 2.0/res );
  ivmax = floor( 2.0/res );
  printf("%s: iumax = %d\n",ME,iumax);
  printf("%s: ivmax = %d\n",ME,ivmax);

  fp = fopen(in_filename,"r");
  if (!fp) {
    printf("%s: FATAL: Couldn't open '%s'\n",ME,in_filename);
    return;
    }

  while (!feof(fp)) {

    fscanf(fp,"%f %f %f",&az,&alt,&t);

    theta = (PI/2.0)-(alt*DTR);
    phi = (PI/2.0)-(az*DTR);
    if (theta<(PI/2.0)) {
      u = cos(phi)*sin(theta);
      v = sin(phi)*sin(theta);
      iu = (int) floor( (u+1.0)/res );
      iv = (int) floor( (v+1.0)/res );
      map[iu][iv] += t;
      count[iu][iv]++;
      } 

    //printf("az=%f alt=%f theta(deg)=%f %f %f %d %d\n",az,alt,theta*180.0/PI,u,v,iu,iv);

    } /* while (!feof(fp)) */

  fclose(fp);

  /* do averaging */
  for (iu=0; iu<iumax; iu++) {
    for (iv=0; iv<ivmax; iv++) {
      if (count[iu][iv]<1) count[iu][iv]=1;
      map[iu][iv] /= count[iu][iv];
      }
    }

  /* open output file */
  fp = fopen("m5.dat","w");
  if (!fp) {
    printf("%s: FATAL: Couldn't open m5.dat\n",ME);
    return;
    }

  /* write it out */
  if (eFormat==0) { /* list */
    for (iv=0; iv<ivmax; iv++) {
      for (iu=0; iu<iumax; iu++) {
        fprintf(fp,"%f %f %f\n", iu*res-1.0, iv*res-1.0, map[iu][iv] );
        }
      }
    }
  if (eFormat==1) { /* matrix */
    for (iv=0; iv<ivmax; iv++) {
      for (iu=0; iu<iumax; iu++) {
        fprintf(fp,"%f ", map[iu][iv] );
        }
      fprintf(fp,"\n" );
      }
    }

  fclose(fp);

  return;
  } /* main() */


